import type { Sequelize } from "sequelize-typescript";
import type { MigrationState } from "./constants";
import { ReverseModelsOptions } from "./utils/getTablesFromModels";
export declare type IMigrationOptions = {
    /**
     * directory where migration file saved. We recommend that you specify this path to sequelize migration path.
     */
    outDir: string;
    /**
     * if true, it doesn't generate files but just prints result action.
     */
    preview?: boolean;
    /**
     * migration file name, default is "noname"
     */
    migrationName?: string;
    /**
     * comment of migration.
     */
    comment?: string;
    debug?: boolean;
} & ReverseModelsOptions;
export declare class SequelizeTypescriptMigration {
    /**
     * generates migration file including up, down code
     * after this, run 'npx sequelize-cli db:migrate'.
     * @param sequelize sequelize-typescript instance
     * @param options options
     */
    static makeMigration: (sequelize: Sequelize, options: IMigrationOptions) => Promise<{
        msg: string;
        code: number;
        info: {
            actions: string[];
            up: string[];
            down: string[];
            previousState: MigrationState;
            currentState: MigrationState;
        };
    }>;
}
